using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using InternshipSystem.Api.Queries;
using InternshipSystem.Core;
using InternshipSystem.Repository;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace InternshipSystem.Api.Controllers
{
    [ApiController]
    [Route("companies")]
    public class CompaniesController : ControllerBase
    {
        public CompaniesController(InternshipDbContext context)
        {
            Context = context;
        }
        private InternshipDbContext Context { get; }

        /// <summary>
        /// Get companies matching provided paginated query
        /// </summary>
        /// <param name="searchQuery">Paginated query description</param>
        /// <returns>Part of companies collection</returns>
        [HttpGet]
        [ProducesResponseType(StatusCodes.Status200OK)]
        public async Task<IReadOnlyCollection<Company>> SearchByNameAsync([FromQuery] SearchQuery searchQuery, CancellationToken cancellationToken) =>
            await Context.Companies
                .Where(c => c.Name.ToLower().Contains(searchQuery.Query.ToLower()))
                .OrderBy(o => o.Name)
                .Skip(searchQuery.Page * searchQuery.PerPage)
                .Take(searchQuery.PerPage)
                .ToListAsync(cancellationToken);
    }

}