using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using InternshipSystem.Core;

namespace InternshipSystem.Repository
{
    public class DatabaseFiller
    {
        public DatabaseFiller(InternshipDbContext context)
        {
            Context = context;
        }

        public InternshipDbContext Context { get; }

        public async Task FillCompany()
        {
            var companies = new List<Company>
            {
                new Company 
                {
                    Name = "Intel",
                    SiteAddress = new Uri("https://www.intel.com/content/www/us/en/jobs/locations/poland.html"),
                    Nip = "9570752316",
                    Range = RangeOfActivities.International,
                    Branches = new List<BranchOffice>
                    {
                        new BranchOffice
                        {
                            Address = new Address
                            {
                                City = "Gdańsk",
                                Street = "ul. Słowackiego",
                                Building = "173",
                                PostalCode = "80-298",
                                Country = "Poland"
                            }
                        },
                        new BranchOffice
                        {
                            Address = new Address
                            {
                                City = "Gdańsk",
                                Street = "Jana z Kolna",
                                Building = "11",
                                PostalCode = "80-001",
                                Country = "Poland"
                            }
                        },
                        new BranchOffice
                        {
                            Address = new Address
                            {
                                City = "Boston",
                                Street = "State St",
                                Building = "53",
                                PostalCode = "MA 02109",
                                Country = "Stany Zjednoczone"
                            }
                        }
                    }
                },
                new Company
                {
                    Name = "Asseco Poland",
                    SiteAddress = new Uri("http://pl.asseco.com"),
                    Nip = "5842068320",
                    Range = RangeOfActivities.National,
                    Branches = new List<BranchOffice>
                    {
                        new BranchOffice
                        {
                            Address = new Address
                            {
                                City = "Gdańsk",
                                Street = "ul. Podolska",
                                Building = "21",
                                PostalCode = "81-321",
                                Country = "Poland"
                            }
                        },
                        new BranchOffice
                        {
                            Address = new Address
                            {
                                City = "Wrocław",
                                Street = "Traugutta",
                                Building = "1/7",
                                PostalCode = "50-449",
                                Country = "Poland"
                            }
                        }
                    }
                }
            };
            Context.Companies.AddRange(companies);
            await Context.SaveChangesAsync();
        } 

        public async Task FillInterns() 
        {
            var interns = new List<Intern>
            {
                new Intern {
                    Name = "Jan",
                    Surname = "Kowalski",
                    AlbumNumber = "123456",
                    Email = "s123456@student.pg.edu.pl",
                    Semester = 4,
                    Course = new Course
                    {
                        Name = "Informatyka",
                        DesiredSemesters = new List<int> { 4, 6 },
                        ProgramEntries = new List<InternshipProgramEntry>
                        {
                            new InternshipProgramEntry
                            {
                                Description = "Instalacja, konfiguracja i administracja niewielkich sieci komputerowych, w tym bezprzewodowych."
                            },
                            new InternshipProgramEntry
                            {
                                Description = "Implementacja polityki bezpieczeństwa informacji w firmie lub instytucji, instalacja ochrony antywirusowej, konfiguracja zapór ogniowych."
                            },
                            new InternshipProgramEntry
                            {
                                Description = "Instalacja, konfiguracja i administracja oprogramowania, w szczególnościsystemów operacyjnychiserwerów aplikacji."
                            },
                            new InternshipProgramEntry
                            {
                                Description = "Projektowanie, implementacja i modyfikacjeoprogramowaniaw różnych technologiach i dla różnych zastosowań."
                            }
                        }
                    }
                },
                new Intern {
                    Name = "Adam",
                    Surname = "Kołek",
                    AlbumNumber = "102137",
                    Email = "s102137@student.pg.edu.pl",
                    Semester = 6,
                    Course = new Course 
                    {
                        Name = "Robotyka",
                        DesiredSemesters = new List<int> { 6 },
                        ProgramEntries = new List<InternshipProgramEntry>
                        {
                            new InternshipProgramEntry
                            {
                                Description = "Testowanie oprogramowania, także z wykorzystaniem narzędzi do testowania automatycznego."
                            },
                            new InternshipProgramEntry
                            {
                                Description = "Wykorzystanie otwartych komponentów programowych z uwzględnieniem prawnych zależności pomiędzy nimi a produktem wynikowym."
                            },
                            new InternshipProgramEntry
                            {
                                Description = "Projektowanie i implementacja baz danych oraz badanie ich wydajności."
                            },
                            new InternshipProgramEntry
                            {
                                Description = "Projektowanie i prototypowaniezaawansowanychinterfejsów użytkownika."
                            }
                        }
                    }
                }
            };
            Context.Interns.AddRange(interns);
            await Context.SaveChangesAsync();
        }

        public async Task FillInternshipTypes() {
            var internshipTypes = new List<InternshipType>
            {
                new InternshipType 
                {
                    Type = "FreeInternship",
                    Description = "Praktyka bezpłatna",
                },
                new InternshipType 
                {
                    Type = "GraduateInternship"
                },
                new InternshipType 
                {
                    Type = "FreeApprenticeship"
                },
                new InternshipType
                {
                    Type = "PaidApprenticeship",
                    Description = "np. przemysłowy"
                },
                new InternshipType
                {
                    Type = "ForeignInternship",
                    Description = "np. IAESTE, ERASMUS"
                },
                new InternshipType
                {
                    Type = "UOP"
                },
                new InternshipType
                {
                    Type = "UD"
                },
                new InternshipType
                {
                    Type = "UZ"
                },
                new InternshipType
                {
                    Type = "Other",
                    Description = "Należy wprowadzić samodzielnie"
                }
            };
            Context.InternshipTypes.AddRange(internshipTypes);
            await Context.SaveChangesAsync();
        }

        public async Task FillEditions() {
            var editions = new List<Edition>
            {
                new Edition 
                {
                    StartDate = new DateTime(2000, 5, 10),
                    EndDate = new DateTime(2000, 11, 10),
                    IPPDeadlineDate = new DateTime(2000, 9, 30),
                    Subjects = new List<InternshipProgramEntry>
                    {
                        Context.InternshipProgramEntries
                            .Where(i => i.Description == "Instalacja, konfiguracja i administracja niewielkich sieci komputerowych, w tym bezprzewodowych.")
                            .First(),
                        Context.InternshipProgramEntries
                            .Where(i => i.Description == "Projektowanie, implementacja i modyfikacjeoprogramowaniaw różnych technologiach i dla różnych zastosowań.")
                            .First()
                    }
                }
            };
            Context.Editions.AddRange(editions);
            await Context.SaveChangesAsync();
        }

        public async Task FillInternShips() {
            var edition = Context.Editions.First();

            var intenrships = new List<Internship>
            {
                new Internship
                {
                    Intern = Context.Interns.Where(i => i.AlbumNumber == "123456").First(),
                    Edition = edition,
                    BranchOffice = 
                        Context.Companies
                            .Where(c => c.Nip == "9570752316")      //Intel
                            .First()
                            .Branches
                            .First(),
                    Type = Context.InternshipTypes.Where(t => t.Type == "UOP").First(),
                    StartDate = new DateTime(2000, 7, 1),
                    EndDate = new DateTime(2000, 8, 30),
                    IsAccepted = false,
                    DeanAcceptance = new DeanAcceptance
                    {
                        AcceptanceDate = new DateTime(2000, 6, 26),
                        IsDeansAcceptanceRequired = true,
                        Reason = DeanAcceptanceReason.Semester
                    },
                    Insurance = new Insurance
                    {
                        InsuranceDeliveryDate = new DateTime(2000, 6, 29),
                        IsInsuranceRequired = true
                    }, 
                    LengthInWeeks = 9,
                    Grade = 0,
                    Program = new List<InternshipProgramEntry>
                    {
                        edition.Subjects.First()
                    },
                    Mentor = new Mentor
                    {
                        Name = "Horacy",
                        Surname = "Wościcki",
                        Email = "howos@intel.com",
                        Phone = "605-555-555"
                    }
                },
                new Internship
                {
                    Intern = Context.Interns.Where(i => i.AlbumNumber == "102137").First(),
                    Edition = edition,
                    BranchOffice = 
                        Context.Companies
                            .Where(c => c.Nip == "5842068320")      //Asseco
                            .First()
                            .Branches
                            .First(),
                    Type = Context.InternshipTypes.Where(t => t.Type == "UZ").First(),
                    StartDate = new DateTime(2000, 7, 1),
                    EndDate = new DateTime(2000, 8, 30),
                    IsAccepted = false,
                    DeanAcceptance = new DeanAcceptance
                    {
                        IsDeansAcceptanceRequired = false
                    },
                    Insurance = new Insurance
                    {
                        InsuranceDeliveryDate = new DateTime(2000, 6, 29),
                        IsInsuranceRequired = true
                    }, 
                    LengthInWeeks = 9,
                    Grade = 0,
                    Program = new List<InternshipProgramEntry>
                    {
                        edition.Subjects.First()
                    },
                    Mentor = new Mentor
                    {
                        Name = "Horacy",
                        Surname = "Wościcki",
                        Email = "howos@intel.com",
                        Phone = "605-555-555"
                    }
                }
            };
            Context.Internships.AddRange(intenrships);
            await Context.SaveChangesAsync();
        }
    } 
}